﻿using System;
using System.Collections;
using System.Threading.Tasks;
using UnityEngine;
using UnityEngine.UI;

public class Type5mManager : CraneManagerV3
{
    // memo: [掴む強さ, 持ち上げた後の強さ, 運ぶ途中の強さ]
    [SerializeField] float[] armLPowerConfig = new float[3]; //アームパワーL(%，未確率時)
    [SerializeField] float[] armLPowerConfigSuccess = new float[3]; //アームパワーL(%，確率時)
    [SerializeField] float[] armRPowerConfig = new float[3]; //アームパワーR(%，未確率時)
    [SerializeField] float[] armRPowerConfigSuccess = new float[3]; //アームパワーR(%，確率時)
    [SerializeField] float armApertures = 80f; //開口率
    [SerializeField] float[] boxRestrictions = new float[2];
    [SerializeField] float downRestriction = 100f;
    public int soundType = 0; //SEの切り替え 0,1: CATCHER 8,9 2: CATCHER 7 Selecterで指定すること
    public float audioPitch = 1f; // サウンドのピッチ(Selecterで指定)
    bool[] isExecuted = new bool[15]; //各craneStatusで1度しか実行しない処理の管理
    public bool buttonPushed = false; //trueならボタンをクリックしているかキーボードを押下している
    public float armLPower;
    public float armRPower;
    [SerializeField] bool player2 = false; //player2の場合true
    [SerializeField] bool button3 = true; //button3の使用可否
    [SerializeField] int[] armSize = new int[2]; // 0:なし，1:S，2:M，3:L
    public Vector2 startPoint; // 開始位置座標定義
    public Vector2 homePoint; // 獲得口座標定義（prizezoneTypeが9のとき使用）
    public int prizezoneType = 9; // 1:左手前，2：左奥，3：右手前，4：右奥，5：左，6：手前，7：右，8：奥，9：特定座標
    TwinArmController armController;
    ArmControllerSupportV3 support;
    ArmUnitLifterV3 lifter;
    ArmNailV3[] nail = new ArmNailV3[2];
    [SerializeField] TextMesh credit3d;
    [SerializeField] TextMesh[] preset = new TextMesh[4];
    public Animator[] animator = new Animator[3];

    [SerializeField] int armMoveSpeed = 3; // アーム移動速度(1～5)
    [SerializeField] int armDownSpeed = 3; // アーム降下速度(1～5)
    [SerializeField] int armUpSpeed = 5; // アーム上昇速度(1～5)

    [SerializeField] float yHomePosition = 0f; // Y座標ホームポジション(1～100)

    private IEnumerator DelayCoroutine(float miliseconds, Action action)
    {
        yield return new WaitForSeconds(miliseconds / 1000f);
        action?.Invoke();
    }

    async void Start()
    {
        Transform temp;
        Transform xLimit = transform.Find("Floor").Find("XLimit");
        Transform zLimit = transform.Find("Floor").Find("ZLimit");
        Transform downLimit = transform.Find("Floor").Find("DownLimit");

        craneStatus = -5;
        craneType = 5;
        // 様々なコンポーネントの取得
        //host = transform.root.Find("CP").GetComponent<MachineHost>();
        canvas = transform.Find("Canvas").gameObject;
        creditSystem = transform.Find("CreditSystem").GetComponent<CreditSystemV3>();
        sp = transform.Find("SE").GetComponent<SEPlayer>();
        getPoint = transform.Find("Floor").Find("GetPoint").GetComponent<GetPointV3>();

        temp = transform.Find("CraneUnit").transform;

        // クレジット情報登録
        // TODO: stub
        int[] priceSet = new int[] {100, 500};
        int[] timesSet = new int[] {1, 6};

        isHibernate = false;
        //creditSystem.rateSet[0, 0] = priceSet[0];
        //creditSystem.rateSet[1, 0] = priceSet[1];
        //creditSystem.rateSet[0, 1] = timesSet[0];
        //creditSystem.rateSet[1, 1] = timesSet[1];
        creditSystem.priceSets = priceSet;
        creditSystem.timesSets = timesSet;
        if (isHibernate)
        {
            creditSystem.SetHibernate();
        }
        else
        {
            preset[0].text = priceSet[0].ToString();
            preset[1].text = priceSet[1].ToString();
            preset[2].text = timesSet[0].ToString();
            preset[3].text = timesSet[1].ToString();
        }

        if (isHibernate | priceSet[1] == 0 || timesSet[1] == 0 || (float)priceSet[0] / timesSet[0] < (float)priceSet[1] / timesSet[1])
        // 未入力の場合，低価格設定を反映 //高額のレートになるとコストが多くなる設定エラーのとき
        {
            if (!player2)
                transform.parent.Find("LCD Component").Find("SegUnit2").gameObject.SetActive(false);// 第2クレジット表示無効に
            else
                transform.parent.Find("LCD Component").Find("SegUnit2 (1)").gameObject.SetActive(false);// 第2クレジット表示無効に
        }

        if (isHibernate)
        {
            if (!player2)
            {
                transform.parent.Find("LCD Component").Find("SegUnit3").gameObject.SetActive(false);
                transform.parent.Find("LCD Component").Find("SegUnit1").gameObject.SetActive(false);
            }
            else
            {
                transform.parent.Find("LCD Component").Find("SegUnit3 (1)").gameObject.SetActive(false);
                transform.parent.Find("LCD Component").Find("SegUnit1 (1)").gameObject.SetActive(false);
            }
        }

        // ロープとアームコントローラに関する処理
        lifter = temp.Find("CraneBox").Find("Tube").Find("TubePoint").GetComponent<ArmUnitLifterV3>();
        armController = temp.Find("ArmUnit").GetComponent<TwinArmController>();
        support = temp.Find("ArmUnit").Find("Main").GetComponent<ArmControllerSupportV3>();

        for (int i = 0; i < 2; i++)
        {
            string a = "Arm" + (i + 1).ToString();
            GameObject arm;
            switch (armSize[i])
            {
                case 0:
                case 1:
                    a += "S";
                    break;
                case 2:
                    a += "M";
                    break;
                case 3:
                    a += "L";
                    break;
            }
            arm = temp.Find("ArmUnit").Find(a).gameObject;
            nail[i] = arm.transform.Find("Nail").GetComponent<ArmNailV3>();
            if (armSize[i] != 0) arm.SetActive(true);
            armController.SetArm(i, armSize[i]);
        }

        await Task.Delay(500);
        // CraneBoxに関する処理
        craneBox = temp.Find("CraneBox").GetComponent<CraneBoxV3>();

        // ロープにマネージャー情報をセット
        creditSystem.Setup();
        creditSystem.SetSEPlayer(sp);
        getPoint.SetManager(this);
        switch (soundType)
        {
            case 0:
            case 1:
            case 2:
                getSoundNum = 7;
                break;
            case 3:
                getSoundNum = 15;
                break;
        }
        lifter.Up(true);

        if (soundType == 0 || soundType == 1 || soundType == 2) creditSystem.SetSoundNum(0);
        else creditSystem.SetSoundNum(8);
        creditSystem.SetSEPlayer(sp);
        support.SetManager(this);
        support.SetLifter(lifter);

        // 効果音のピッチを設定する
        // Selectorから移動
        sp.SetAudioPitch(audioPitch);
        sp.SetAudioPitch(audioPitch);

        support.pushTime = 300; // 押し込みパワーの調整
        await Task.Delay(500);
        for (int i = 0; i < 2; i++)
        {
            nail[i].SetManager(this);
            nail[i].SetLifter(lifter);
        }

        for (int i = 0; i < 15; i++)
            isExecuted[i] = false;

        if (!button3)
        {
            transform.Find("Canvas").Find("ControlGroup").Find("Button 3").gameObject.SetActive(false);
            transform.Find("Floor").Find("Button3").gameObject.SetActive(false);
        }

        // イニシャル移動とinsertFlagを後に実行
        while (!lifter.UpFinished())
        {
            await Task.Delay(100);
        }
        armController.SetLimit(armApertures);

        // アーム移動速度設定
        // 開始位置への移動前に設定すること
        this.SetArmMoveSpeed(armMoveSpeed);
        this.SetArmLiftSpeed(armUpSpeed, armDownSpeed);

        // pos + 0.01
        // 元:-0.31f 仮:-0.235f
        if (!player2)
        {
            startPoint = new Vector2(-0.61f + startPoint.x, -0.235f + startPoint.y);
            homePoint = new Vector2(-0.61f + homePoint.x, -0.235f + homePoint.y);
            if (boxRestrictions[0] < 100) xLimit.localPosition = new Vector3(-0.5f + 0.004525f * boxRestrictions[0], xLimit.localPosition.y, xLimit.localPosition.z);

            // 移動範囲制限
            craneBox.SetLimits(-0.61f + 0.004525f * boxRestrictions[0], -0.61f, -0.19f + 0.00605f * boxRestrictions[1], -0.235f);
        }
        else
        {
            startPoint = new Vector2(0.61f - startPoint.x, -0.235f + startPoint.y);
            homePoint = new Vector2(0.61f - homePoint.x, -0.235f + homePoint.y);
            if (boxRestrictions[0] < 100) xLimit.localPosition = new Vector3(0.5f - 0.004525f * boxRestrictions[0], xLimit.localPosition.y, xLimit.localPosition.z);

            // 移動範囲制限
            craneBox.SetLimits(0.61f, 0.61f - 0.004525f * boxRestrictions[0], -0.19f + 0.00605f * boxRestrictions[1], -0.235f);
        }
        if (boxRestrictions[1] < 100) zLimit.localPosition = new Vector3(zLimit.localPosition.x, zLimit.localPosition.y, -0.19f + 0.00605f * boxRestrictions[1]);
        if (downRestriction < 100) downLimit.localPosition = new Vector3(downLimit.localPosition.x, 1.4f - 0.005975f * downRestriction, downLimit.localPosition.z);
        if (yHomePosition > 0 && yHomePosition <= 100)
        {
            yHomePosition = -0.206f - 0.604f * (yHomePosition / 100.0f);
        }
        craneBox.GoPosition(startPoint);

        host.manualCode = 7;
        craneStatus = -4;

        //StartCoroutine(Init());

    }

    IEnumerator Init()
    {
        Debug.Log("Starting...");
        yield return new WaitForSeconds(3);

        // TODO: イニシャライズ実装(3パターン)
        // 原点に移動
        craneBox.GoPosition(startPoint);

        // アーム開閉
        armController.Open();
        yield return new WaitForSeconds(1);
        armController.Close(100f);
        yield return new WaitForSeconds(1);

        // 中央に移動
        // TODO: 中央到達チェック未実装
        if (!player2) {
            craneBox.Right(true);
        } else {
            craneBox.Left(true);
        }
        craneBox.Back(true);

        // 奥に移動

        while (true)
        {
            if (craneBox.CheckPos(!player2 ? 4 : 2)) break;
            yield return null;
        }

        // 下降
        if (!player2) {
            craneBox.Right(false);
        } else {
            craneBox.Left(false);
        }
        craneBox.Back(false);
        lifter.Down(true);
        while (true)
        {
            if (lifter.DownFinished()) break;
            yield return null;
        }
        yield return new WaitForSeconds(1);

        // 上昇
        lifter.Up(true);
        while (true)
        {
            if (lifter.UpFinished()) break;
            yield return null;
        }
        yield return new WaitForSeconds(1);

        // 原点に移動
        // TODO: 実装
        if (!player2) {
            craneBox.Left(true);
        } else {
            craneBox.Right(true);
        }
        craneBox.Forward(true);

        // ホームポジションに移動
        while (true)
        {
            if (craneBox.CheckPos(!player2 ? 1 : 3)) break;
            yield return null;
        }
        if (!player2) {
            craneBox.Left(false);
        } else {
            craneBox.Right(false);
        }
        craneBox.Forward(false);

        StartCoroutine(Setup());
    }

    IEnumerator Setup()
    {
        if (craneBox.CheckPos(9))
        {
            yield return null;
        }

        craneStatus = 0;
    }

    void Update()
    {
        // 常時実行しないといけない処理だけ書く
        //   →位置チェックなど
        // 1回だけ実行すればいいものはFirstStatusEventに書く
        credit3d.text = craneStatus.ToString();
        if (useUI && host.playable && !canvas.activeSelf) canvas.SetActive(true);
        else if (!host.playable && canvas.activeSelf) canvas.SetActive(false);
        if (!player2 && (Input.GetKeyDown(KeyCode.Keypad0) || Input.GetKeyDown(KeyCode.Alpha0))) InsertCoin();
        else if (player2 && (Input.GetKeyDown(KeyCode.KeypadPeriod) || Input.GetKeyDown(KeyCode.Minus))) InsertCoin();

        if (craneStatus == -4 && ((craneBox.CheckPos(1) && !player2) || (craneBox.CheckPos(3) && player2)))
        {
            craneStatus = -3;
        }
        if (craneStatus == -3)
        {
            if (craneBox.CheckPos(9))
            {
                craneStatus = yHomePosition != 0 ? -2 : 0;
            }
        }
        else if (craneStatus == -2)
        {
            if (lifter.DownFinished())
            {
                craneStatus = -1;
            }
        }
        else if (craneStatus == -1)
        {
            craneStatus = 0;
        }

        // キー操作
        if (craneStatus > 0 && craneStatus <= 6)
        {
            DetectKey(craneStatus);
        }

        if (craneStatus == 0)
        {
        }
        else if (craneStatus == 1)
        {
            // コイン投入
        }
        else if (craneStatus == 2)
        {
            // 右移動中
            // 移動制限に達したら強制的に終了
            if (!player2 && craneBox.CheckPos(7))
            {
                buttonPushed = false;
                craneStatus = 3;
            }
            if (player2 && craneBox.CheckPos(5))
            {
                buttonPushed = false;
                craneStatus = 3;
            }

            // 右移動効果音ループ再生;
        }
        else if (craneStatus == 3)
        {
            // 右移動終了

        }
        else if (craneStatus == 4)
        {
            // 奥移動中
            if (craneBox.CheckPos(8))
            {
                buttonPushed = false;
                craneStatus = 5;
            }
        }
        else if (craneStatus == 6)
        {
            // アーム下降が終了した
            if (lifter.DownFinished())
            {
                craneStatus = 7;
            }
        }
        else if (craneStatus == 8)
        {
            if (lifter.UpFinished())
            {
                craneStatus = 9;
            }
        }
        else if (craneStatus == 10)
        {
            if (craneBox.CheckPos(prizezoneType))
            {
                craneStatus = 11;
            }
        }
        else if (craneStatus == 13)
        {
            if ((craneBox.CheckPos(1) && !player2) || (craneBox.CheckPos(3) && player2))
            {
                craneStatus = 14;
            }
        }
        else if (craneStatus == 15)
        {
            // ホームポジションへの移動
            if (craneBox.CheckPos(9))
            {
                craneStatus = 16;
            }
        }
        else if (craneStatus == 17)
        {
            if (lifter.GoPositionFinished())
            {
                craneStatus = 18;
            }
        }
    }

    void FixedUpdate()
    {
        if (craneStatus != 0)
        {
            if (craneStatus == 8)
            {
                if (probability)
                {
                    if (armLPower > armLPowerConfigSuccess[1]) armLPower -= 0.15f;
                    if (armRPower > armRPowerConfigSuccess[1]) armRPower -= 0.15f;
                }
                else
                {
                    if (armLPower > armLPowerConfig[1]) armLPower -= 0.15f;
                    if (armRPower > armRPowerConfig[1]) armRPower -= 0.15f;
                }
                armController.SetMotorPower(armLPower, 0);
                armController.SetMotorPower(armRPower, 1);
            }
            else if (craneStatus == 10 || craneStatus == 15)
            {
                if (craneStatus == 10)
                {
                    if (probability)
                    {
                        if (armLPower > armLPowerConfigSuccess[2]) armLPower -= 0.15f;
                        if (armRPower > armRPowerConfigSuccess[2]) armRPower -= 0.15f;
                    }
                    else
                    {
                        if (armLPower > armLPowerConfig[2]) armLPower -= 0.15f;
                        if (armRPower > armRPowerConfig[2]) armRPower -= 0.15f;
                    }
                    armController.SetMotorPower(armLPower, 0);
                    armController.SetMotorPower(armRPower, 1);
                }
            }
        }
    }

    public override void GetPrize()
    {
        for (int i = 0; i < 3; i++) animator[i].SetTrigger("GetPrize");
        base.GetPrize();
    }

    protected override void DetectKey(int num)
    {
        if (host.playable)
        {
            switch (num)
            {
                case 1:
                    if ((Input.GetKeyDown(KeyCode.Keypad1) || Input.GetKeyDown(KeyCode.Alpha1)) && !buttonPushed && !player2)
                    {
                        ButtonDown(1);
                    }
                    if ((Input.GetKeyDown(KeyCode.Keypad7) || Input.GetKeyDown(KeyCode.Alpha7)) && !buttonPushed && player2)
                    {
                        ButtonDown(4);
                    }
                    break;
                //投入を無効化
                case 2:
                    if ((Input.GetKeyUp(KeyCode.Keypad1) || Input.GetKeyUp(KeyCode.Alpha1)) && buttonPushed && !player2)
                    {
                        ButtonUp(1);
                    }
                    if ((Input.GetKeyUp(KeyCode.Keypad7) || Input.GetKeyUp(KeyCode.Alpha7)) && buttonPushed && player2)
                    {
                        ButtonUp(4);
                    }
                    break;
                case 3:
                    if ((Input.GetKeyDown(KeyCode.Keypad2) || Input.GetKeyDown(KeyCode.Alpha2)) && !buttonPushed && !player2)
                    {
                        ButtonDown(2);
                    }
                    if ((Input.GetKeyDown(KeyCode.Keypad8) || Input.GetKeyDown(KeyCode.Alpha8)) && !buttonPushed && player2)
                    {
                        ButtonDown(2);
                    }
                    break;
                case 4:
                    if ((Input.GetKeyUp(KeyCode.Keypad2) || Input.GetKeyUp(KeyCode.Alpha2)) && buttonPushed && !player2)
                    {
                        ButtonUp(2);
                    }
                    if ((Input.GetKeyUp(KeyCode.Keypad8) || Input.GetKeyUp(KeyCode.Alpha8)) && buttonPushed && player2)
                    {
                        ButtonUp(2);
                    }
                    break;
                case 6:
                    if ((Input.GetKeyDown(KeyCode.Keypad3) || Input.GetKeyDown(KeyCode.Alpha3)) && !player2 && button3)
                    {
                        ButtonDown(3);
                    }
                    if ((Input.GetKeyDown(KeyCode.Keypad9) || Input.GetKeyDown(KeyCode.Alpha9)) && player2 && button3)
                    {
                        ButtonDown(3);
                    }
                    break;
            }
        }
    }

    public override void ButtonDown(int num)
    {
        switch (num)
        {
            case 1:
                if (craneStatus == 1 && !buttonPushed)
                {
                    buttonPushed = true;
                    craneStatus = 2;
                }
                break;
            case 2:
                if ((craneStatus == 3 && !buttonPushed) || (craneStatus == 4 && buttonPushed))
                {
                    buttonPushed = true;
                    craneStatus = 4;
                }
                break;
            case 3:
                if (craneStatus == 6)
                {
                    lifter.Down(false);
                    craneStatus = 7;
                }
                break;
            case 4: // player2 case 1:
                if (craneStatus == 1 && !buttonPushed)
                {
                    buttonPushed = true;
                    craneStatus = 2;
                }
                break;
        }
    }

    public override void ButtonUp(int num)
    {
        switch (num)
        {
            case 1:
                if (craneStatus == 2 && buttonPushed)
                {
                    craneStatus = 3;
                    buttonPushed = false;
                }
                break;
            case 2:
                if (craneStatus == 4 && buttonPushed)
                {
                    craneStatus = 5;
                    buttonPushed = false;
                }
                break;
            case 4: // player2 case 1:
                if (craneStatus == 2 && buttonPushed)
                {
                    craneStatus = 3;
                    buttonPushed = false;
                }
                break;
        }
    }

    public override void InsertCoin()
    {
        if (!isHibernate && host.playable && craneStatus >= 0)
        {
            int credit = creditSystem.Pay(100);
            if (credit < 100) credit3d.text = credit.ToString();
            else credit3d.text = "99.";
            if (credit > 0 && craneStatus == 0) craneStatus = 1;
        }
    }

    public override void InsertCoinAuto()
    {
        if (!isHibernate && craneStatus >= 0)
        {
            int credit = creditSystem.Pay(100);
            if (credit < 100) credit3d.text = credit.ToString();
            else credit3d.text = "99.";
            if (credit > 0 && craneStatus == 0) craneStatus = 1;
        }
    }

    protected override void FirstStatusEvent(int status)
    {
        switch (status)
        {
            case -4:
                // 起動後 原点移動
                if (!player2)
                {
                    craneBox.Left(true);
                }
                else
                {
                    craneBox.Right(true);
                }
                craneBox.Forward(true);
                break;

            case -3:
                // 起動後 ホームポジション移動
                // 移動を停止
                craneBox.Left(false);
                craneBox.Right(false);
                craneBox.Forward(false);

                // ホームポジションへ移動
                // TODO: ウェイト必要なら入れる
                craneBox.GoPosition(homePoint);
                break;

            case -2:
                // Y軸ホームポジションへ移動
                // 移動
                lifter.GoPosition(yHomePosition);
                break;

            case -1:
                // Y軸ホームポジションへ移動終了
                break;

            case 2:
                // 右方向移動中
                // 確率カウント
                creditSystem.ResetPay();
                creditSystem.NewPlay();

                // クレジットの消費
                int credit = creditSystem.DecrementCredit();
                credit3d.text = credit < 100 ? credit.ToString() : "99.";

                // 確率抽選
                probability = creditSystem.ProbabilityCheck();
                Debug.Log("Probability:" + probability);

                // 移動
                if (!player2)
                {
                    craneBox.Right(true);
                }
                else
                {
                    craneBox.Left(true);
                }

                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(1);
                        break;
                    case 3:
                        sp.Play(9);
                        break;
                }
                break;

            case 3:
                // 右方向移動終了
                if (!player2)
                {
                    craneBox.Right(false);
                }
                else
                {
                    craneBox.Left(false);
                }

                // 効果音を停止する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Stop(1);
                        break;
                    case 3:
                        sp.Stop(9);
                        break;
                }
                break;

            case 4:
                // 奥方向移動中
                craneBox.Back(true);

                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(2);
                        break;

                    case 3:
                        sp.Play(10);
                        break;
                }
                break;

            case 5:
                // 奥移動効果音を停止
                sp.Stop(1);

                // 奥方向移動終了
                craneBox.Back(false);

                int waitTime = 1700;

                // アームを開く
                //armController.Open();
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        armController.Open();
                        sp.Stop(2);
                        sp.Play(3, 1);
                        break;
                    case 3:
                        armController.Open(-45f);
                        sp.Stop(10);
                        sp.Play(11, 1);
                        break;
                }

                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Stop(2);
                        sp.Play(3, 1);
                        break;
                    case 3:
                        sp.Stop(10);
                        sp.Play(11, 1);
                        break;
                }

                // 下降開始してstatusを進める
                // TODO: type3の再利用なので間違ってるかも
                StartCoroutine(DelayCoroutine(waitTime, () =>
                {
                    if (craneStatus == 5) craneStatus = 6;
                }));
                break;

            case 6:
                // 下降中
                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(4);
                        break;
                    case 3:
                        sp.Play(12);
                        break;
                }
                // アーム下降を開始する
                lifter.Down(true);
                break;

            case 7:
                // 下降停止
                // 下降の効果音を停止する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Stop(4);
                        break;
                    case 3:
                        sp.Stop(12);
                        break;
                }
                int downWaitTime = 1000;
                StartCoroutine(DelayCoroutine(downWaitTime, () =>
                {
                    // 掴む効果音を再生する
                    switch (soundType)
                    {
                        case 0:
                        case 1:
                        case 2:
                            sp.Play(5, 1);
                            break;
                        case 3:
                            sp.Play(13, 1);
                            break;
                    }

                    // アームを閉じる
                    //armController.Close(30f);
                    switch (soundType)
                    {
                        case 0:
                        case 1:
                        case 2:
                            sp.Play(5, 1);
                            armController.Close(30f);
                            break;
                        case 3:
                            sp.Play(13, 1);
                            armController.Close(30f, 22f);
                            break;
                    }

                    // 効果音と同じタイミングでパワーを入れる
                    // TODO: 閉じ速度に影響ある?
                    if (probability)
                    {
                        if (armLPowerConfigSuccess[0] >= 30) armLPower = armLPowerConfigSuccess[0];
                        else armLPower = 30f;
                        if (armRPowerConfigSuccess[0] >= 30) armRPower = armRPowerConfigSuccess[0];
                        else armRPower = 30f;
                    }
                    else
                    {
                        if (armLPowerConfig[0] >= 30) armLPower = armLPowerConfig[0];
                        else armLPower = 30f;
                        if (armRPowerConfig[0] >= 30) armRPower = armRPowerConfig[0];
                        else armRPower = 30f;
                    }
                    armController.SetMotorPower(armLPower, 0);
                    armController.SetMotorPower(armRPower, 1);

                    int closeWaitTime = 3000;
                    StartCoroutine(DelayCoroutine(closeWaitTime, () =>
                    {
                        craneStatus = 8;
                    }));
                }));
                break;

            case 8:
                // アーム上昇開始
                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(4);
                        break;
                    case 3:
                        sp.Play(14);
                        break;
                }

                // アームを上昇する
                lifter.Up(true);
                break;

            case 9:
                // アーム上昇停止
                // 効果音を停止する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Stop(4);
                        break;
                    case 3:
                        sp.Stop(14);
                        break;
                }
                StartCoroutine(DelayCoroutine(200, () =>
                {
                    // アームパワーを決定する
                    if (probability)
                    {
                        armLPower = armLPowerConfigSuccess[1];
                        armRPower = armLPowerConfigSuccess[1];
                    }
                    else
                    {
                        armLPower = armLPowerConfig[1];
                        armRPower = armRPowerConfig[1];
                    }

                    // アームパワーを設定する
                    armController.SetMotorPower(armLPower, 0);
                    armController.SetMotorPower(armRPower, 1);

                    craneStatus = 10;
                }));
                break;

            case 10:
                // アーム獲得口ポジションへ移動
                // 任意座標への移動はここで設定
                if (prizezoneType == 9)
                {
                    craneBox.GoPosition(homePoint);
                }
                else
                {
                    switch (prizezoneType) // 1:左手前，2：左奥，3：右手前，4：右奥，5：左，6：手前，7：右，8：奥，9：特定座標
                    {
                        case 1:
                            craneBox.Left(true);
                            craneBox.Forward(true);
                            break;
                        case 2:
                            craneBox.Left(true);
                            craneBox.Back(true);
                            break;
                        case 3:
                            craneBox.Right(true);
                            craneBox.Forward(true);
                            break;
                        case 4:
                            craneBox.Right(true);
                            craneBox.Back(true);
                            break;
                        case 5:
                            craneBox.Left(true);
                            break;
                        case 6:
                            craneBox.Forward(true);
                            break;
                        case 7:
                            craneBox.Right(true);
                            break;
                        case 8:
                            craneBox.Back(true);
                            break;
                    }
                }

                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(6);
                        break;
                    case 3:
                        sp.Play(9);
                        break;
                }
                break;

            case 11:
                // アーム開放
                // 移動を停止させる
                craneBox.Left(false);
                craneBox.Right(false);
                craneBox.Back(false);
                craneBox.Forward(false);

                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Stop(6);
                        sp.Play(3, 1);
                        break;
                    case 3:
                        sp.Stop(9);
                        sp.Play(11, 1);
                        break;
                }
                // アームを開く
                //armController.Open();
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Stop(6);
                        sp.Play(3, 1);
                        armController.Open();
                        break;
                    case 3:
                        sp.Stop(9);
                        sp.Play(11, 1);
                        armController.Open(-45f);
                        break;
                }

                StartCoroutine(DelayCoroutine(2500, () =>
                {
                    craneStatus = 12;
                }));
                break;

            case 12:
                // アーム閉じる
                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(5, 1);
                        break;
                    case 3:
                        sp.Play(13, 1);
                        break;
                }
                // アームを閉じる
                //armController.Close(100f);
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(5, 1);
                        armController.Close(100f);
                        break;
                    case 3:
                        sp.Play(13, 1);
                        armController.Close(100f, 22f);
                        break;
                }

                // 1秒(->2.45秒)待機した後に遷移
                StartCoroutine(DelayCoroutine(2450, () =>
                {
                    craneStatus = 13;
                }));
                break;

            case 13:
                // アーム初期位置へ移動
                // 移動
                // TODO: ここのGoPositionだけモーター音が停止しない
                //craneBox.GoPosition(homePoint);
                if (!player2)
                {
                    craneBox.Left(true);
                }
                else
                {
                    craneBox.Right(true);
                }
                craneBox.Forward(true);

                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(6);
                        break;
                }
                break;

            case 14:
                // アーム初期位置へ移動終了
                // 移動停止
                // TODO: ここで移動音が停止していない
                craneBox.Left(false);
                craneBox.Right(false);
                craneBox.Back(false);
                craneBox.Forward(false);

                // 効果音を停止する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Stop(6);
                        break;
                }

                // 14 -> 1秒待機した後に15に遷移
                // TODO: 待機時間を実機で調べる
                StartCoroutine(DelayCoroutine(1000, () =>
                {
                    craneStatus = 15;
                }));
                break;

            case 15:
                // ホームポジションへ移動
                // 移動
                craneBox.GoPosition(startPoint);

                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(6);
                        break;
                }
                break;

            case 16:
                // ホームポジション移動終了
                // 移動停止
                craneBox.Left(false);
                craneBox.Right(false);
                craneBox.Back(false);
                craneBox.Forward(false);

                // 移動音を停止
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Stop(6);
                        break;
                }

                // TODO: 待機時間を実機で調べる
                StartCoroutine(DelayCoroutine(500, () =>
                {
                    craneStatus = yHomePosition != 0 ? 17 : 18;
                }));
                break;

            case 17:
                // Y軸ホームポジションへ移動
                // 移動
                lifter.GoPosition(yHomePosition);

                // 効果音を再生する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Play(4);
                        break;
                }
                break;

            case 18:
                // Y軸ホームポジションへ移動終了
                // 効果音を停止する
                switch (soundType)
                {
                    case 0:
                    case 1:
                    case 2:
                        sp.Stop(4);
                        break;
                }

                // TODO: 待機時間を実機で調べる
                StartCoroutine(DelayCoroutine(1000, () =>
                {
                    // 残りクレジットがあればstatusを1に、なければ0に
                    craneStatus = creditSystem.GetCreditCount() > 0 ? 1 : 0;
                }));
                break;
        }
    }

    protected override void LastStatusEvent(int status)
    {

    }

    /// <summary>
    /// アームの移動速度を設定する
    /// </summary>
    /// <param name="_moveSpeed">移動速度</param>
    private void SetArmMoveSpeed(int _moveSpeed)
    {
        // 移動速度配列
        float[] speedDelta = new float[] { 0.002f, 0.0025f, 0.003f, 0.0035f, 0.004f };

        // 設定する移動速度
        float setSpeed;
        switch (_moveSpeed)
        {
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
                setSpeed = speedDelta[_moveSpeed - 1];
                break;
            default:
                setSpeed = speedDelta[2];
                break;
        }

        // 移動速度を設定する
        craneBox.SetMoveSpeeds(setSpeed, setSpeed);
    }

    /// <summary>
    /// アームの昇降速度を設定する
    /// </summary>
    /// <param name="_upSpeed">アームの上昇速度</param>
    /// <param name="_downSpeed">アームの降下速度</param>
    private void SetArmLiftSpeed(int _upSpeed, int _downSpeed)
    {
        // 昇降速度配列
        float[] speedDelta = new float[] { 0.002f, 0.0025f, 0.003f, 0.0035f, 0.004f };

        // 設定する上昇速度
        float setUpSpeed;
        switch (_upSpeed)
        {
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
                setUpSpeed = speedDelta[_upSpeed - 1];
                break;
            default:
                setUpSpeed = speedDelta[2];
                break;
        }

        // 設定する降下速度
        float setDownSpeed;
        switch (_downSpeed)
        {
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
                setDownSpeed = speedDelta[_downSpeed - 1];
                break;
            default:
                setDownSpeed = speedDelta[2];
                break;
        }

        // 昇降速度を設定する
        lifter.upSpeed = setUpSpeed;
        lifter.downSpeed = setDownSpeed;
    }
}
